--[[
	Simple HUD!
	Copyright (C) 2011  Christopher 'Kasoki' Kaster

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]--

---------------------------
--    global variables   --
---------------------------

local context = UI.CreateContext("Context")

local SIMPLE_HUD_BAR_WIDTH = 25
local SIMPLE_HUD_BAR_HEIGHT = 250
local SIMPLE_HUD_BAR_Y_OFFSET = {
	DEFAULT = 50,
	TEXT = 50 - 135,
	COMBO = 200
}
local SIMPLE_HUD_BAR_X_OFFSET = {
	HEALTH = 200,
	ENERGY = 235,
	POWER = 235,
	MANA = 235,
	CHARGE = 270
}
local SIMPLE_HUD_BAR_COLOR = {
	HEALTH = {
		R = 0,
		G = 1,
		B = 0,
		A = 0.5
	},
	ENERGY = {
		R = 1,
		G = 0,
		B = 1,
		A = 0.5
	},
	POWER = {
		R = 1,
		G = 1,
		B = 0,
		A = 0.5
	},
	MANA = {
		R = 0,
		G = 0,
		B = 1,
		A = 0.5
	},
	CHARGE = {
		R = 0,
		G = 1,
		B = 1,
		A = 0.5
	}
}
local SIMPLE_HUD_BAR_TYPE = {
	HEALTH = 0x00,
	ENERGY = 0x01,
	POWER = 0x02,
	MANA = 0x03,
	CHARGE = 0x04,
	COMBO = 0x05,
	CAST = 0x06
}
local CLASS = {
	WARRIOR = "warrior",
	ROGUE = "rogue",
	MAGE = "mage",
	CLERIC = "cleric",
	NONE = nil
}
local COMBO_MAX = 0

local player, target
local playerHealth, playerResource, playerCharge, playerCombo, playerChargeLoaded = false
local targetCalling, targetHealth, targetResource

---------------------------
--     event handler     --
---------------------------

function update()
	local player = Inspect.Unit.Detail("player")
	local target = Inspect.Unit.Detail("player.target")
	
	if not player then
		return
	end
	
	playerHealth:refresh(player)
	playerResource:refresh(player)
	
	if player.calling == CLASS.MAGE then
		playerCharge:refresh(player)
		
		if not playerCharge:getVisible() and not playerChargeLoaded then
			playerCharge:setVisible(true)
			playerChargeLoaded = true
		end
	elseif player.calling == CLASS.WARRIOR then
		COMBO_MAX = 3
		playerCombo:refresh(player)
		
		if not playerCombo:getVisible() and not playerCombo:getHidden() then
			playerCombo:setVisible(true)
		end
	elseif player.calling == CLASS.ROGUE then
		COMBO_MAX = 5
		playerCombo:refresh(player)
		
		if not playerCombo:getVisible() and not playerCombo:getHidden() then
			playerCombo:setVisible(true)
		end
	end
	
	if target then		
		targetHealth:setVisible(true)
		targetResource:setVisible(true)
		
		targetHealth:refresh(target)
		targetResource:refresh(target)
	else
		targetHealth:setVisible(false)
		targetResource:setVisible(false)
	end
end

function setup()
	if setupLoaded then
		return
	end
	
	setupLoaded = true
	
	playerHealth = createIndicatorBar(SIMPLE_HUD_BAR_TYPE.HEALTH, true)
	playerResource = createIndicatorBar(SIMPLE_HUD_BAR_TYPE.ENERGY, true)
	playerCharge = createIndicatorBar(SIMPLE_HUD_BAR_TYPE.CHARGE, true)
	playerCombo = createComboText()
	
	playerCharge:setVisible(false)
	playerCombo:setVisible(false)
	
	targetHealth = createIndicatorBar(SIMPLE_HUD_BAR_TYPE.HEALTH, false)
	targetResource = createIndicatorBar(SIMPLE_HUD_BAR_TYPE.POWER, false)
end

--------------------------
--    frame functions   --
--------------------------

function createIndicatorBar(barType, isPlayer)
	local bar = {}
	bar.value = 0
	bar.last_percent = 0
	
	-- build bar
	bar.frame = UI.CreateFrame("Texture", "simplehud.frame.bar", context)
	bar.bgframe = UI.CreateFrame("Texture", "simplehud.frame.bgbar", context)
	bar.text = UI.CreateFrame("Text", "simplehud.frame.bartext", context)
	
	bar.frame:SetHeight(SIMPLE_HUD_BAR_HEIGHT)
	bar.frame:SetWidth(SIMPLE_HUD_BAR_WIDTH)
	
	bar.text:SetText("100%")
	
	-- background
	bar.bgframe:SetHeight(SIMPLE_HUD_BAR_HEIGHT)
	bar.bgframe:SetWidth(SIMPLE_HUD_BAR_WIDTH)
	
	-- text
	bar.text:SetHeight(bar.text:GetFullHeight())
	bar.text:SetWidth(bar.text:GetFullWidth())

	-- other local vars
	bar.xoffset = 0
	bar.yoffset = 0
	bar.type = barType
	
	if barType == SIMPLE_HUD_BAR_TYPE.HEALTH then
		bar.xoffset = SIMPLE_HUD_BAR_X_OFFSET.HEALTH
		bar.yoffset = SIMPLE_HUD_BAR_Y_OFFSET.DEFAULT
		bar.yoffset_text = SIMPLE_HUD_BAR_Y_OFFSET.TEXT
		bar.color_r = SIMPLE_HUD_BAR_COLOR.HEALTH.R
		bar.color_g = SIMPLE_HUD_BAR_COLOR.HEALTH.G
		bar.color_b = SIMPLE_HUD_BAR_COLOR.HEALTH.B
	elseif barType == SIMPLE_HUD_BAR_TYPE.ENERGY then
		bar.xoffset = SIMPLE_HUD_BAR_X_OFFSET.ENERGY
		bar.yoffset = SIMPLE_HUD_BAR_Y_OFFSET.DEFAULT
		bar.yoffset_text = SIMPLE_HUD_BAR_Y_OFFSET.TEXT
		bar.color_r = SIMPLE_HUD_BAR_COLOR.ENERGY.R
		bar.color_g = SIMPLE_HUD_BAR_COLOR.ENERGY.G
		bar.color_b = SIMPLE_HUD_BAR_COLOR.ENERGY.B
	elseif barType == SIMPLE_HUD_BAR_TYPE.POWER then
		bar.xoffset = SIMPLE_HUD_BAR_X_OFFSET.POWER
		bar.yoffset = SIMPLE_HUD_BAR_Y_OFFSET.DEFAULT
		bar.yoffset_text = SIMPLE_HUD_BAR_Y_OFFSET.TEXT
		bar.color_r = SIMPLE_HUD_BAR_COLOR.POWER.R
		bar.color_g = SIMPLE_HUD_BAR_COLOR.POWER.G
		bar.color_b = SIMPLE_HUD_BAR_COLOR.POWER.B
	elseif barType == SIMPLE_HUD_BAR_TYPE.MANA then
		bar.xoffset = SIMPLE_HUD_BAR_X_OFFSET.MANA
		bar.yoffset = SIMPLE_HUD_BAR_Y_OFFSET.DEFAULT
		bar.yoffset_text = SIMPLE_HUD_BAR_Y_OFFSET.TEXT
		bar.color_r = SIMPLE_HUD_BAR_COLOR.MANA.R
		bar.color_g = SIMPLE_HUD_BAR_COLOR.MANA.G
		bar.color_b = SIMPLE_HUD_BAR_COLOR.MANA.B
	elseif barType == SIMPLE_HUD_BAR_TYPE.CHARGE then
		bar.xoffset = SIMPLE_HUD_BAR_X_OFFSET.CHARGE
		bar.yoffset = SIMPLE_HUD_BAR_Y_OFFSET.DEFAULT
		bar.yoffset_text = SIMPLE_HUD_BAR_Y_OFFSET.TEXT
		bar.color_r = SIMPLE_HUD_BAR_COLOR.CHARGE.R
		bar.color_g = SIMPLE_HUD_BAR_COLOR.CHARGE.G
		bar.color_b = SIMPLE_HUD_BAR_COLOR.CHARGE.B
	end
	
	if isPlayer then
		bar.xoffset = bar.xoffset * -1
	end
	
	bar.text:SetPoint("CENTERCENTER", UIParent, "CENTERCENTER",
		bar.xoffset, bar.yoffset_text
	)

	bar.bgframe:SetBackgroundColor(0, 0, 0, 0.3)
	bar.bgframe:SetPoint("CENTERCENTER", UIParent, "CENTERCENTER",
		bar.xoffset, bar.yoffset
	)
	
	bar.frame:SetBackgroundColor(
		bar.color_r,
		bar.color_g,
		bar.color_b,
		0.5
	)
	bar.frame:SetPoint("CENTERCENTER", UIParent, "CENTERCENTER",
		bar.xoffset, bar.yoffset
	) -- y offset -width/2
	
	-- bar methods
	bar.refresh = function(self, unit)
		if not unit then
			return
		end

		percent = 0
	
		if self.type == SIMPLE_HUD_BAR_TYPE.HEALTH then
			percent = math.floor(unit.health / unit.healthMax * 100)
		elseif self.type == SIMPLE_HUD_BAR_TYPE.CHARGE then
			percent = unit.charge
		else
			if unit.calling == CLASS.WARRIOR then
				percent = math.floor(unit.power / 100 * 100)
				self.setColor(self, SIMPLE_HUD_BAR_COLOR.POWER)
			elseif unit.calling == CLASS.ROGUE then
				percent = math.floor(unit.energy / unit.energyMax * 100)
				self.setColor(self, SIMPLE_HUD_BAR_COLOR.ENERGY)
			elseif (unit.calling == CLASS.MAGE) or (unit.calling == CLASS.CLERIC) then
				percent = math.floor(unit.mana / unit.manaMax * 100)
				self.setColor(self, SIMPLE_HUD_BAR_COLOR.MANA)
			else
				self.setVisible(self, false)
			end		
		end
		
		if percent == 100 then
			self.setVisible(self, false)
			
			if self.type == SIMPLE_HUD_BAR_TYPE.CHARGE then
				self.setVisible(self, true)
			end
		else
			if not (unit.calling == CLASS.NONE) then
				self.setVisible(self, true)
			end
			
			if unit.player then
				if unit.calling == CLASS.MAGE then
					if percent == 0 then
						self.setVisible(self, false)
					else
						self.setVisible(self, true)
					end
				end
			end
			self.reduceBar(self, percent)
		end
		
		self.text:SetText(percent.."%")
		
		self.text:SetHeight(self.text:GetFullHeight())
		self.text:SetWidth(self.text:GetFullWidth())
	end
	
	bar.setValue = function(self, value)
		self.value = value
	end
	
	bar.reduceBar = function(self, percent)
		if not (self.last_percent == percent) then
			percent = 100 - percent
			barPercent = SIMPLE_HUD_BAR_HEIGHT * (percent/100)
			self.frame:SetHeight(SIMPLE_HUD_BAR_HEIGHT - barPercent)
			self.text:SetPoint("CENTERCENTER", UIParent, "CENTERCENTER", self.xoffset, self.yoffset_text + barPercent)
			self.frame:SetPoint("CENTERCENTER", UIParent, "CENTERCENTER", self.xoffset, self.yoffset + (barPercent/2))
		end
		
		self.last_percent = percent
	end
	
	bar.setColor = function(self, colorType)
		self.frame:SetBackgroundColor(
			colorType.R, colorType.G, colorType.B, colorType.A
		)
	end
	
	bar.setVisible = function(self, visible)
		bar.frame:SetVisible(visible)
		bar.bgframe:SetVisible(visible)
		bar.text:SetVisible(visible)
	end
	
	bar.getVisible = function(self)
		return bar.frame:GetVisible()
	end
	
	return bar
end

function createComboText()
	combo = {}
	combo.hidden = false
	
	combo.frame = UI.CreateFrame("Text", "simplehud.frame.combotext", context)
	
	combo.frame:SetFontSize(30)
	--combo.frame:SetFontColor(1, 0, 0, 1)
	combo.frame:SetText("0")
	
	combo.frame:SetHeight(combo.frame:GetFullHeight())
	combo.frame:SetWidth(combo.frame:GetFullWidth())
	
	combo.frame:SetPoint("CENTERCENTER", UIParent, "CENTERCENTER", 0, SIMPLE_HUD_BAR_Y_OFFSET.COMBO)
	
	-- combo methods
	combo.setVisible = function(self, visible)
		combo.frame:SetVisible(visible)
	end
	
	combo.getVisible = function(self)
		return combo.frame:GetVisible()
	end
	
	combo.getHidden = function(self)
		return self.hidden
	end
	
	combo.refresh = function(self, unit)
		self.frame:SetText(""..unit.combo)
		
		if unit.combo == COMBO_MAX then
			self.frame:SetFontColor(1, 0, 0, 1)
			self.frame:SetFontSize(50)
			self.frame:SetHeight(self.frame:GetFullHeight())
			self.frame:SetWidth(self.frame:GetFullWidth())
		else
			if unit.combo == 0 then
				self.frame:SetVisible(false)
				self.hidden = true
			else
				self.frame:SetVisible(true)
				self.hidden = false
			end
			
			if not (self.frame:GetFontSize() == 30) then
				self.frame:SetFontColor(1, 1, 1, 1)
				self.frame:SetFontSize(30)
			end
		end
	end
	
	return combo
end

---------------------------
-- important functions ----
---------------------------

function init()
	table.insert(Event.System.Update.Begin, {update, "simplehud", "simplehud.update"})
	table.insert(Event.Addon.Load.End, {setup, "simplehud", "simplehud.setup"})
end

---------------------------
---        init        ----
---------------------------

init()
print(__VERSION.." loaded.")